using System.Collections.Generic;

namespace ClieOp.Record
{
    internal class RecordLayout
    {
        private static RecordLayout _instance;
        private readonly Dictionary<long, RecordType> _codeToType;
        private readonly Dictionary<RecordType, FieldLayout> _layout;

        private RecordLayout()
        {
            _layout = new Dictionary<RecordType, FieldLayout>
                          {
                              {
                                  RecordType.Batchsluitrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 9990),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Totaalbedrag, FieldType.Type9, 18),
                                          new Field(FieldName.Totaalrekeningnummers, FieldType.Type9, 10),
                                          new Field(FieldName.Aantalposten, FieldType.Type9, 7),
                                          new Field(FieldName.Filler, FieldType.TypeX, 10, "")
                                      }
                                  },
                              {
                                  RecordType.Batchvoorlooprecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0010),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "B"),
                                          new Field(FieldName.Transactiegroep, FieldType.TypeX, 2),
                                          new Field(FieldName.Rekeningnummeropdrachtgever, FieldType.Type9, 10),
                                          new Field(FieldName.Batchvolgnummer, FieldType.Type9, 4),
                                          new Field(FieldName.Aanleveringsmuntsoort, FieldType.TypeX, 3),
                                          new Field(FieldName.Filler, FieldType.TypeX, 26, "")
                                      }
                                  },
                              {
                                  RecordType.Bestandssluitrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 9999),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Filler, FieldType.TypeX, 45, "")
                                      }
                                  },
                              {
                                  RecordType.Bestandsvoorlooprecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0001),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Aanmaakdatumbestand, FieldType.Type9, 6),
                                          new Field(FieldName.Bestandsnaam, FieldType.TypeX, 8, "CLIEOP03"),
                                          new Field(FieldName.Inzendersidentificatie, FieldType.TypeX, 5),
                                          new Field(FieldName.Bestandsidentificatie, FieldType.TypeX, 4),
                                          new Field(FieldName.Duplicaatcode, FieldType.Type9, 1),
                                          new Field(FieldName.Filler, FieldType.TypeX, 21, "")
                                      }
                                  },
                              {
                                  RecordType.Betalingskenmerkrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0150),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Betalingskenmerk, FieldType.TypeX, 16),
                                          new Field(FieldName.Filler, FieldType.TypeX, 29, "")
                                      }
                                  },
                              {
                                  RecordType.Naambegunstigderecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0170),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "B"),
                                          new Field(FieldName.Naambegunstigde, FieldType.TypeX, 35),
                                          new Field(FieldName.Filler, FieldType.TypeX, 10, "")
                                      }
                                  },
                              {
                                  RecordType.Naambetalerrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0110),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "B"),
                                          new Field(FieldName.Naambetaler, FieldType.TypeX, 35),
                                          new Field(FieldName.Filler, FieldType.TypeX, 10, "")
                                      }
                                  },
                              {
                                  RecordType.Omschrijvingsrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0160),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Omschrijving, FieldType.TypeX, 32),
                                          new Field(FieldName.Filler, FieldType.TypeX, 13, "")
                                      }
                                  },
                              {
                                  RecordType.Opdrachtgeverrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0030),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "B"),
                                          new Field(FieldName.Nawcode, FieldType.Type9, 1),
                                          new Field(FieldName.Gewensteverwerkingsdatum, FieldType.Type9, 6),
                                          new Field(FieldName.Naamopdrachtgever, FieldType.TypeX, 35),
                                          new Field(FieldName.Testcode, FieldType.TypeX, 1),
                                          new Field(FieldName.Filler, FieldType.TypeX, 2, "")
                                      }
                                  },
                              {
                                  RecordType.Transactierecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0100),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Transactiesoort, FieldType.TypeX, 4),
                                          new Field(FieldName.Bedrag, FieldType.Type9, 12),
                                          new Field(FieldName.Rekeningnummerbetaler, FieldType.Type9, 10),
                                          new Field(FieldName.Rekeningnummerbegunstigde, FieldType.Type9, 10),
                                          new Field(FieldName.Filler, FieldType.TypeX, 9, "")
                                      }
                                  },
                              {
                                  RecordType.Vasteomschrijvingsrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0020),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "A"),
                                          new Field(FieldName.Vasteomschrijving, FieldType.TypeX, 32),
                                          new Field(FieldName.Filler, FieldType.TypeX, 13, "")
                                      }
                                  },
                              {
                                  RecordType.Woonplaatsrecord,
                                  new FieldLayout
                                      {
                                          new Field(FieldName.Recordcode, FieldType.Type9, 4, 0173),
                                          new Field(FieldName.Variantcode, FieldType.TypeX, 1, "B"),
                                          new Field(FieldName.Filler, FieldType.TypeX, 45)
                                      }
                                  }
                          };
            _codeToType = new Dictionary<long, RecordType>();
            foreach (var definition in _layout)
            {
                _codeToType.Add(definition.Value.Recordcode, definition.Key);
            }
        }

        public static RecordLayout Instance
        {
            get { return _instance ?? (_instance = new RecordLayout()); }
        }

        public RecordType this[long key]
        {
            get { return _codeToType[key]; }
        }

        public FieldLayout this[RecordType key]
        {
            get { return _layout[key]; }
        }

        public bool ContainsKey(long key)
        {
            return _codeToType.ContainsKey(key);
        }
    }
}